itk::usual MOS {}

namespace eval mos {

   class CONV_MC {

      constructor { args } { eval configure $args }
      destructor { }

      public proc coeff_mc { mx1 my1 cx1 cy1 mx2 my2 cx2 cy2 mx3 my3 cx3 cy3  }
      public proc c2m { cx cy }
      public proc m2c { mx my }
      public proc test { }

      public variable eqx 2000.0

      private common f1 0.0
      private common f2 0.0
      private common f3 0.0
      private common f4 0.0
      private common p1 0.0
      private common p2 0.0
      private common f5 0.0
      private common f6 0.0
      private common f7 0.0
      private common f8 0.0
      private common p3 0.0
      private common p4 0.0

   }
   # end of the class CONV_MC


#------------------------------------------------------------------------------------------
   ::itcl::body CONV_MC::coeff_mc { mx1 my1 cx1 cy1 mx2 my2 cx2 cy2 mx3 my3 cx3 cy3  } {

      # This method creates the matrix for conversion between mm focal plane and canvas
      # three points are given, each one in twoo coord systems m for mm focal plane and c for canvas
      # the coord conversion matrix elements are common variables and accessable to all instances of mos_slit

      #--------------------------------------------------------------------
      # from canvas to mm focal plane

      # now we have to extrapolate using some minor vector algebra
      # (mx)    (p1)   (f1 f2)   (cx)
      # (  ) =  (  ) + (     ) * (  )
      # (my)    (p2)   (f3 f4)   (cy)

      # create the matrix elements f1-f4, f1 and f3 have the same nominator
      set nom1 [expr ($cx1-$cx2)*($cy2-$cy3)-($cy1-$cy2)*($cx2-$cx3)]
      set f1 [expr ($mx1-$mx2)*($cy2-$cy3)-($mx2-$mx3)*($cy1-$cy2)]
      set f3 [expr ($my1-$my2)*($cy2-$cy3)-($my2-$my3)*($cy1-$cy2)]
      set f1 [expr $f1/$nom1]; set f3 [expr $f3/$nom1];
      # f2 and f4 have the same nominator; nom1 = -nom2
      set nom2 [expr ($cy1-$cy2)*($cx2-$cx3)-($cy2-$cy3)*($cx1-$cx2)]
      set f2 [expr ($mx1-$mx2)*($cx2-$cx3)-($mx2-$mx3)*($cx1-$cx2)]
      set f4 [expr ($my1-$my2)*($cx2-$cx3)-($my2-$my3)*($cx1-$cx2)]
      set f2 [expr $f2/$nom2];  set f4 [expr $f4/$nom2]

      # now the offset
      set p1 [expr $mx1 - $cx1*$f1-$cy1*$f2]
      set p2 [expr $my1 - $cx1*$f3-$cy1*$f4]

      #-------------------------------------------------------------
      # from mm to canvas

      # now we have to extrapolate using some minor vector algebra
      # (cx)    (p3)   (f5 f6)   (mx)
      # (  ) =  (  ) + (     ) * (  )
      # (cy)    (p4)   (f7 f8)   (my)

      # create the matrix elements f5-f8, f5 and f7 have the same nominator
      set nom1 [expr ($mx1-$mx2)*($my2-$my3)-($my1-$my2)*($mx2-$mx3)]
      set f5 [expr ($cx1-$cx2)*($my2-$my3)-($cx2-$cx3)*($my1-$my2)]
      set f7 [expr ($cy1-$cy2)*($my2-$my3)-($cy2-$cy3)*($my1-$my2)]
      set f5 [expr $f5/$nom1]; set f7 [expr $f7/$nom1];
      # f6 and f8 have the same nominator
      set nom2 [expr ($my1-$my2)*($mx2-$mx3)-($my2-$my3)*($mx1-$mx2)]
      set f6 [expr ($cx1-$cx2)*($mx2-$mx3)-($cx2-$cx3)*($mx1-$mx2)]
      set f8 [expr ($cy1-$cy2)*($mx2-$mx3)-($cy2-$cy3)*($mx1-$mx2)]
      set f6 [expr $f6/$nom2];  set f8 [expr $f8/$nom2]

      # now the offset
      set p3 [expr $cx1 - $mx1*$f5-$my1*$f6]
      set p4 [expr $cy1 - $mx1*$f7-$my1*$f8]

   }


#------------------------------------------------------------------------------------------
   ::itcl::body CONV_MC::c2m { cx cy } {

      # converts canvas coords to mm focplane coords
      return [list [expr $p1+$f1*$cx+$f2*$cy] [expr $p2+$f3*$cx+$f4*$cy] ]

   }

#------------------------------------------------------------------------------------------
   ::itcl::body CONV_MC::m2c { mx my } {

      # converts mm focplane coords to canvas coords
      return [list [expr $p3+$mx*$f5+$my*$f6] [expr $p4+$mx*$f7+$my*$f8] ]

   }

#------------------------------------------------------------------------------------------
   ::itcl::body CONV_MC::test { } {

      # shown common variables
      puts stdout "$p1 $p2 $p3 $p4 $f1 $f2 $f3 $f4 $f5 $f6"

   }

}
# end of namespace
